package com.rjconsultores.ventaboletos.web.gui.controladores.catalogos;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.event.EventListener;
import org.zkoss.zul.Paging;

import com.rjconsultores.ventaboletos.entidad.EmbarqueCatracaRodoviaria;
import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.service.EmpresaService;
import com.rjconsultores.ventaboletos.web.utilerias.MyComboboxEmpresa;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.MyTextbox;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.HibernateSearchObject;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.PagedListWrapper;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderConfiguracaoCodBarras;

@Controller("busquedaConfiguracaoCodBarrasController")
@Scope("prototype")
public class BusquedaConfiguracaoCodBarrasController extends MyGenericForwardComposer {

	private static final long serialVersionUID = 1L;

	@Autowired
	private transient PagedListWrapper<EmbarqueCatracaRodoviaria> pageListEmbarqueCatraca;
	@Autowired
	private EmpresaService empresaService;

	private MyListbox codBarrasList;
	private MyTextbox txtSequencia;
	private MyTextbox txtEmpresaRod;
	private MyComboboxEmpresa comboEmpresa;

	private Paging pagingConfiguracaoCodBarras;

	private List<Empresa> lsEmpresa;

	@Override
	public void doAfterCompose(Component comp) throws Exception {
		lsEmpresa = empresaService.obtenerTodos();
		super.doAfterCompose(comp);
		codBarrasList.setItemRenderer(new RenderConfiguracaoCodBarras());
		codBarrasList.addEventListener("onDoubleClick", new EventListener() {

			@Override
			public void onEvent(Event arg0) throws Exception {
				EmbarqueCatracaRodoviaria embarqueCatraca = (EmbarqueCatracaRodoviaria) codBarrasList.getSelected();
				verCodigoBarra(embarqueCatraca);
			}
		});

		refresh();
	}

	public void refresh() {
		HibernateSearchObject<EmbarqueCatracaRodoviaria> codBarrasBusqueda = new HibernateSearchObject<EmbarqueCatracaRodoviaria>(EmbarqueCatracaRodoviaria.class, pagingConfiguracaoCodBarras.getPageSize());

		if (txtSequencia.getValue() != null && !txtSequencia.getValue().isEmpty()) {
			codBarrasBusqueda.addFilterEqual("sequenciaRod", new Integer(txtSequencia.getValue()));
		}

		if (txtEmpresaRod.getValue() != null && !txtEmpresaRod.getValue().isEmpty()) {
			codBarrasBusqueda.addFilterEqual("empresaRod", new Integer(txtEmpresaRod.getValue()));
		}

		if (comboEmpresa.getSelectedItem() != null) {
			Empresa empresa = (Empresa) comboEmpresa.getSelectedItem().getValue();
			codBarrasBusqueda.addFilterEqual("empresa.nombempresa", empresa.getNombempresa());
		}

		codBarrasBusqueda.addSortAsc("sequenciaRod");
		codBarrasBusqueda.addFilterEqual("activo", true);

		pageListEmbarqueCatraca.init(codBarrasBusqueda, codBarrasList, pagingConfiguracaoCodBarras);
	}

	private void verCodigoBarra(EmbarqueCatracaRodoviaria embarqueCatracaRodoviaria) {
		Map<String, Object> args = new HashMap<String, Object>();
		args.put("embarqueCatraca", embarqueCatracaRodoviaria);
		args.put("codBarrasList", codBarrasList);
		openWindow("/gui/catalogos/editarConfiguracaoCodBarras.zul",
				Labels.getLabel("editarConfiguracaoCodBarrasController.title"), args, MODAL);
	}

	public void onClick$btnNovo(Event event) {
		verCodigoBarra(null);
	}

	public void onClick$btnRefresh(Event event) {
		refresh();
	}

	public void onClick$btnPesquisa(Event event) {
		refresh();
	}

	public List<Empresa> getLsEmpresa() {
		return lsEmpresa;
	}

	public void setLsEmpresa(List<Empresa> lsEmpresa) {
		this.lsEmpresa = lsEmpresa;
	}

}
